"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var couchbase_plugin_common_1 = require("./couchbase-plugin.common");
var types = require("tns-core-modules/utils/types");
var couchbase_plugin_common_2 = require("./couchbase-plugin.common");
exports.QueryMeta = couchbase_plugin_common_2.QueryMeta;
exports.QueryArrayOperator = couchbase_plugin_common_2.QueryArrayOperator;
exports.QueryLogicalOperator = couchbase_plugin_common_2.QueryLogicalOperator;
var Couchbase = (function (_super) {
    __extends(Couchbase, _super);
    function Couchbase(databaseName) {
        var _this = _super.call(this, databaseName) || this;
        _this.ios = CBLDatabase.alloc().initWithNameError(databaseName);
        return _this;
    }
    Couchbase.prototype.createDocument = function (data, documentId) {
        var doc;
        if (documentId) {
            doc = CBLMutableDocument.alloc().initWithID(documentId);
        }
        else {
            doc = CBLMutableDocument.alloc().init();
        }
        var keys = Object.keys(data);
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            var item = data[key];
            this.serialize(item, doc, key);
        }
        this.ios.saveDocumentError(doc);
        return doc.id;
    };
    Couchbase.prototype.fromISO8601UTC = function (date) {
        var dateFormatter = NSDateFormatter.new();
        dateFormatter.dateFormat = 'yyyy-MM-dd\'T\'HH:mm:ss.SSSZ';
        return dateFormatter.dateFromString(date);
    };
    Couchbase.prototype.serializeObject = function (item, object, key) {
        var _this = this;
        switch (typeof item) {
            case 'object':
                if (item instanceof Date) {
                    object.setDateForKey(this.fromISO8601UTC(item.toISOString()), key);
                    return;
                }
                if (Array.isArray(item)) {
                    var array_1 = CBLMutableArray.new();
                    item.forEach(function (data) {
                        _this.serializeArray(data, array_1);
                    });
                    object.setArrayForKey(array_1, key);
                    return;
                }
                var nativeObject_1 = CBLMutableDictionary.new();
                Object.keys(item).forEach(function (itemKey) {
                    var obj = item[itemKey];
                    _this.serializeObject(obj, nativeObject_1, itemKey);
                });
                object.setDictionaryForKey(object, key);
                break;
            case 'number':
                object.setIntegerForKey(item, key);
                break;
            case 'boolean':
                object.setBooleanForKey(item, key);
                break;
            default:
                object.setValueForKey(item, key);
        }
    };
    Couchbase.prototype.serializeArray = function (item, array) {
        var _this = this;
        switch (typeof item) {
            case 'object':
                if (item instanceof Date) {
                    array.addDate(this.fromISO8601UTC(item.toISOString()));
                    return;
                }
                if (Array.isArray(item)) {
                    var nativeArray_1 = CBLMutableArray.new();
                    item.forEach(function (data) {
                        _this.serializeArray(data, nativeArray_1);
                    });
                    array.addArray(nativeArray_1);
                    return;
                }
                var object_1 = CBLMutableDictionary.new();
                Object.keys(item).forEach(function (itemKey) {
                    var obj = item[itemKey];
                    _this.serializeObject(obj, object_1, itemKey);
                });
                array.addDictionary(object_1);
                break;
            case 'number':
                array.addInteger(item);
                break;
            case 'boolean':
                array.addBoolean(item);
                break;
            default:
                array.addValue(item);
        }
    };
    Couchbase.prototype.serialize = function (item, doc, key) {
        var _this = this;
        switch (typeof item) {
            case 'object':
                if (item instanceof Date) {
                    doc.setDateForKey(this.fromISO8601UTC(item.toISOString()), key);
                    return;
                }
                if (Array.isArray(item)) {
                    var array_2 = CBLMutableArray.new();
                    item.forEach(function (data) {
                        _this.serializeArray(data, array_2);
                    });
                    doc.setArrayForKey(array_2, key);
                    return;
                }
                var object_2 = CBLMutableDictionary.new();
                Object.keys(item).forEach(function (itemKey) {
                    var obj = item[itemKey];
                    _this.serializeObject(obj, object_2, itemKey);
                });
                doc.setDictionaryForKey(object_2, key);
                break;
            case 'number':
                doc.setIntegerForKey(item, key);
                break;
            case 'boolean':
                doc.setBooleanForKey(item, key);
                break;
            default:
                doc.setValueForKey(item, key);
        }
    };
    Couchbase.prototype.getDocument = function (documentId) {
        var doc = this.ios.documentWithID(documentId);
        if (doc) {
            var obj = {};
            var keys = doc.keys;
            var size = keys.count;
            obj['id'] = doc.id;
            for (var i = 0; i < size; i++) {
                var key = keys.objectAtIndex(i);
                var value = doc.valueForKey(key);
                var newValue = {};
                newValue[key] = this.deserialize(value);
                obj = Object.assign(obj, newValue);
            }
            return obj;
        }
        return null;
    };
    Couchbase.prototype.deserialize = function (data) {
        if (typeof data === 'string' || typeof data === 'number' || typeof data === 'boolean' || typeof data !== 'object')
            return data;
        if (types.isNullOrUndefined(data)) {
            return data;
        }
        if (data instanceof NSNull) {
            return null;
        }
        if (data instanceof CBLDictionary) {
            var keys = data.keys;
            var length_1 = keys.count;
            var object = {};
            for (var i = 0; i < length_1; i++) {
                var key = keys.objectAtIndex(i);
                var nativeItem = data.valueForKey(key);
                object[key] = this.deserialize(nativeItem);
            }
            return object;
        }
        if (data instanceof CBLArray) {
            var array = [];
            var size = data.count;
            for (var i = 0; i < size; i++) {
                var nativeItem = data.valueAtIndex(i);
                var item = this.deserialize(nativeItem);
                array.push(item);
            }
            return array;
        }
        return data;
    };
    Couchbase.prototype.updateDocument = function (documentId, data) {
        var original = this.ios.documentWithID(documentId);
        var newDoc = original.toMutable();
        var keys = Object.keys(data);
        for (var _i = 0, keys_2 = keys; _i < keys_2.length; _i++) {
            var key = keys_2[_i];
            var item = data[key];
            newDoc.setValueForKey(item, key);
        }
        this.ios.saveDocumentError(newDoc);
    };
    Couchbase.prototype.deleteDocument = function (documentId) {
        var doc = this.ios.documentWithID(documentId);
        return this.ios.deleteDocumentError(doc);
    };
    Couchbase.prototype.destroyDatabase = function () {
        return this.ios.delete();
    };
    Couchbase.prototype.createPullReplication = function (remoteUrl) {
        var url = NSURL.alloc().initWithString(remoteUrl);
        var targetEndpoint = CBLURLEndpoint.alloc().initWithURL(url);
        var replConfig = CBLReplicatorConfiguration.alloc().initWithDatabaseTarget(this.ios, targetEndpoint);
        replConfig.replicatorType = CBLReplicatorType.kCBLReplicatorTypePull;
        var replicator = CBLReplicator.alloc().initWithConfig(replConfig);
        return new Replicator(replicator);
    };
    Couchbase.prototype.createPushReplication = function (remoteUrl) {
        var url = NSURL.alloc().initWithString(remoteUrl);
        var targetEndpoint = CBLURLEndpoint.alloc().initWithURL(url);
        var replConfig = CBLReplicatorConfiguration.alloc().initWithDatabaseTarget(this.ios, targetEndpoint);
        replConfig.replicatorType = CBLReplicatorType.kCBLReplicatorTypePush;
        var replicator = CBLReplicator.alloc().initWithConfig(replConfig);
        return new Replicator(replicator);
    };
    Couchbase.prototype.addDatabaseChangeListener = function (callback) {
        this.ios.addChangeListener(function (change) {
            if (callback && typeof callback === 'function') {
                var ids = [];
                var documentIds = change.documentIDs;
                var size = documentIds.count;
                for (var i = 0; i < size; i++) {
                    var item = documentIds[i];
                    ids.push(item);
                }
                callback(ids);
            }
        });
    };
    Couchbase.prototype.setComparision = function (item) {
        var nativeQuery;
        switch (item.comparison) {
            case 'equalTo':
                nativeQuery = CBLQueryExpression.property(item.property).equalTo(CBLQueryExpression.value(item.value));
                break;
            case 'add':
                nativeQuery = CBLQueryExpression.property(item.property).add(CBLQueryExpression.value(item.value));
                break;
            case 'between':
                if (Array.isArray(item.value) && item.value.length === 2) {
                    nativeQuery = CBLQueryExpression.property(item.property).between(CBLQueryExpression.value(item.value[0])).andExpression(CBLQueryExpression.value(item.value[1]));
                }
                break;
            case 'collate':
                nativeQuery = CBLQueryExpression.property(item.property).collate(CBLQueryExpression.value(item.value));
                break;
            case 'divide':
                nativeQuery = CBLQueryExpression.property(item.property).divide(CBLQueryExpression.value(item.value));
                break;
            case 'greaterThan':
                nativeQuery = CBLQueryExpression.property(item.property).greaterThan(CBLQueryExpression.value(item.value));
                break;
            case 'greaterThanOrEqualTo':
                nativeQuery = CBLQueryExpression.property(item.property).greaterThanOrEqualTo(CBLQueryExpression.value(item.value));
                break;
            case 'in':
                var inArray = [];
                if (Array.isArray(item.value)) {
                    for (var _i = 0, _a = item.value; _i < _a.length; _i++) {
                        var exp_1 = _a[_i];
                        inArray.push(CBLQueryExpression.value(exp_1));
                    }
                }
                else {
                    inArray.push(CBLQueryExpression.value(item.value));
                }
                nativeQuery = CBLQueryExpression.property(item.property).in(inArray);
                break;
            case 'is':
                nativeQuery = CBLQueryExpression.property(item.property).is(CBLQueryExpression.value(item.value));
                break;
            case 'isNot':
                nativeQuery = CBLQueryExpression.property(item.property).isNot(CBLQueryExpression.value(item.value));
                break;
            case 'isNullOrMissing':
                nativeQuery = CBLQueryExpression.property(item.property).isNullOrMissing();
                break;
            case 'lessThan':
                nativeQuery = CBLQueryExpression.property(item.property).lessThan(CBLQueryExpression.value(item.value));
                break;
            case 'lessThanOrEqualTo':
                nativeQuery = CBLQueryExpression.property(item.property).lessThanOrEqualTo(CBLQueryExpression.value(item.value));
                break;
            case 'like':
                nativeQuery = CBLQueryExpression.property(item.property).like(CBLQueryExpression.value(item.value));
                break;
            case 'modulo':
                nativeQuery = CBLQueryExpression.property(item.property).modulo(CBLQueryExpression.value(item.value));
                break;
            case 'multiply':
                nativeQuery = CBLQueryExpression.property(item.property).multiply(CBLQueryExpression.value(item.value));
                break;
            case 'notEqualTo':
                nativeQuery = CBLQueryExpression.property(item.property).notEqualTo(CBLQueryExpression.value(item.value));
                break;
            case 'notNullOrMissing':
                nativeQuery = CBLQueryExpression.property(item.property).notNullOrMissing();
                break;
            case 'regex':
                nativeQuery = CBLQueryExpression.property(item.property).regex(CBLQueryExpression.value(item.value));
                break;
        }
        return nativeQuery;
    };
    Couchbase.prototype.query = function (query) {
        if (query === void 0) { query = { select: [couchbase_plugin_common_1.QueryMeta.ALL, couchbase_plugin_common_1.QueryMeta.ID] }; }
        var items = [];
        var select = [];
        var from;
        var join = null;
        var where;
        var groupBy = null;
        var orderBy = null;
        var limit = null;
        var having = null;
        if (!query.select || query.select.length === 0) {
            select.push(CBLQuerySelectResult.all());
            select.push(CBLQuerySelectResult.expression(CBLQueryMeta.id()));
        }
        else {
            query.select.forEach(function (item) {
                if (item === couchbase_plugin_common_1.QueryMeta.ID) {
                    select.push(CBLQuerySelectResult.expression(CBLQueryMeta.id()));
                }
                else if (item === couchbase_plugin_common_1.QueryMeta.ALL) {
                    select.push(CBLQuerySelectResult.all());
                }
                else {
                    select.push(CBLQueryExpression.property(item));
                }
            });
        }
        if (query.from) {
            var db = new Couchbase(query.from);
            from = CBLQueryDataSource.database(db.ios);
        }
        else {
            from = CBLQueryDataSource.database(this.ios);
        }
        var nativeQuery = null;
        if (query.where) {
            for (var _i = 0, _a = query.where; _i < _a.length; _i++) {
                var item = _a[_i];
                if (item === couchbase_plugin_common_1.QueryLogicalOperator.AND) {
                    if (!nativeQuery)
                        break;
                    nativeQuery.andExpression(this.setComparision(item));
                }
                else if (item === couchbase_plugin_common_1.QueryLogicalOperator.OR) {
                    if (!nativeQuery)
                        break;
                    nativeQuery.orExpression(this.setComparision(item));
                }
                else {
                    nativeQuery = this.setComparision(item);
                }
            }
        }
        if (query.groupBy) {
            if (query.groupBy.length > 0) {
                groupBy = [];
            }
            for (var _b = 0, _c = query.groupBy; _b < _c.length; _b++) {
                var prop = _c[_b];
                groupBy.push(CBLQueryExpression.property(prop));
            }
        }
        if (query.order) {
            if (query.order.length > 0) {
                orderBy = [];
            }
            for (var _d = 0, _e = query.order; _d < _e.length; _d++) {
                var item = _e[_d];
                switch (item.direction) {
                    case 'desc':
                        orderBy.push(CBLQueryOrdering.property(item.property).descending());
                        break;
                    default:
                        orderBy.push(CBLQueryOrdering.property(item.property).ascending());
                        break;
                }
            }
        }
        if (query.limit && typeof query.limit === 'number') {
            limit = CBLQueryLimit.limit(CBLQueryExpression.value(query.limit));
        }
        var queryBuilder = CBLQueryBuilder.selectFromJoinWhereGroupByHavingOrderByLimit(select, from, join, nativeQuery, groupBy, having, orderBy, limit);
        var result = queryBuilder.execute().allResults();
        var size = result.count;
        for (var i = 0; i < size; i++) {
            var item = result.objectAtIndex(i);
            var keys = item.keys;
            var keysSize = keys.count;
            var obj = {};
            for (var keyId = 0; keyId < keysSize; keyId++) {
                var key = keys.objectAtIndex(keyId);
                var nativeItem = item.valueForKey(key);
                if (typeof nativeItem === 'string') {
                    obj[key] = nativeItem;
                }
                else if (types.getClass(nativeItem) === 'CBLDictionary') {
                    var cblKeys = nativeItem.keys;
                    var cblKeysSize = cblKeys.count;
                    for (var cblKeysId = 0; cblKeysId < cblKeysSize; cblKeysId++) {
                        var cblKey = cblKeys.objectAtIndex(cblKeysId);
                        obj[cblKey] = this.deserialize(nativeItem.valueForKey(cblKey));
                    }
                }
            }
            items.push(obj);
        }
        return items;
    };
    return Couchbase;
}(couchbase_plugin_common_1.Common));
exports.Couchbase = Couchbase;
var Replicator = (function (_super) {
    __extends(Replicator, _super);
    function Replicator(replicator) {
        return _super.call(this, replicator) || this;
    }
    Replicator.prototype.start = function () {
        this.replicator.start();
    };
    Replicator.prototype.stop = function () {
        this.replicator.stop();
    };
    Replicator.prototype.isRunning = function () {
        return (this.replicator.status.activity ===
            CBLReplicatorActivityLevel.kCBLReplicatorBusy);
    };
    Replicator.prototype.setContinuous = function (isContinuous) {
        var newConfig = CBLReplicatorConfiguration.alloc().initWithConfig(this.replicator.config);
        newConfig.continuous = isContinuous;
        this.replicator = CBLReplicator.alloc().initWithConfig(newConfig);
    };
    Replicator.prototype.setUserNameAndPassword = function (username, password) {
        var newConfig = CBLReplicatorConfiguration.alloc().initWithConfig(this.replicator.config);
        newConfig.authenticator = CBLBasicAuthenticator.alloc().initWithUsernamePassword(username, password);
        this.replicator = CBLReplicator.alloc().initWithConfig(newConfig);
    };
    Replicator.prototype.setSessionIdAndCookieName = function (sessionId, cookieName) {
        var newConfig = CBLReplicatorConfiguration.alloc().initWithConfig(this.replicator.config);
        newConfig.authenticator = CBLSessionAuthenticator.alloc().initWithSessionIDCookieName(sessionId, cookieName);
        this.replicator = CBLReplicator.alloc().initWithConfig(newConfig);
    };
    Replicator.prototype.setSessionId = function (sessionId) {
        var newConfig = CBLReplicatorConfiguration.alloc().initWithConfig(this.replicator.config);
        newConfig.authenticator = CBLSessionAuthenticator.alloc().initWithSessionID(sessionId);
        this.replicator = CBLReplicator.alloc().initWithConfig(newConfig);
    };
    return Replicator;
}(couchbase_plugin_common_1.ReplicatorBase));
exports.Replicator = Replicator;
//# sourceMappingURL=couchbase-plugin.ios.js.map